`timescale 1ns / 1ps

module play1(bclk,
	     a,
	     d,
	     cs3,
	     cs4,
	     rd,
	     wr0,
	     wr1,
	     waitn,
	     edack0,
	     edreq0,

	     vclk,
	     vcke,
	     vcs,
	     vba,
	     vras,
	     vcas,
	     vwe,
	     va,
	     vd,
	     vdqmh,
	     vdqml,

	     dvi_red_n,
	     dvi_red_p,
	     dvi_green_n,
	     dvi_green_p,
	     dvi_blue_n,
	     dvi_blue_p,
	     dvi_clock_p,
	     dvi_clock_n,
	     dvi_ddc_clock,
	     dvi_ddc_data,
	     dvi_hotplug,
	     );

   input   	bclk;
   input [23:0] a;
   inout [15:0] d;
   input 	cs3;
   input 	cs4;
   input 	rd;
   input 	wr0;
   input 	wr1;
   output 	waitn;
   input 	edack0;
   output 	edreq0;
   
   output 	vclk;
   output 	vcke;
   output 	vcs;
   output [1:0] vba;
   output 	vras;
   output 	vcas;
   output 	vwe;
   output [11:0] va;
   output [15:0] vd;
   output 	 vdqmh;
   output 	 vdqml;
   
   output 	 dvi_red_n;
   output 	 dvi_red_p;
   output 	 dvi_green_n;
   output 	 dvi_green_p;
   output 	 dvi_blue_n;
   output 	 dvi_blue_p;
   output 	 dvi_clock_p;
   output 	 dvi_clock_n;
   output 	 dvi_ddc_clock;
   output 	 dvi_ddc_data;
   input 	 dvi_hotplug;
   
   wire 	 dvi_red_n;
   wire 	 dvi_red_p;
   wire 	 dvi_green_n;
   wire 	 dvi_green_p;
   wire 	 dvi_blue_n;
   wire 	 dvi_blue_p;
   wire 	 dvi_clock_p;
   wire 	 dvi_clock_n;
   reg 		 dvi_ddc_clock;
   reg 		 dvi_ddc_data;
   
   
   reg [15:0] 	 d_wr;
   wire 		 d_en;
   assign d = d_en ? d_wr : 16'bz;

   reg 		 my_wait = 0;
   assign waitn = my_wait ? 1 : 0;
   
   assign edreq0 = 0;
   
   assign vclk = 0;
   assign vcke = 0;
   assign vcs = 1;
   assign vba = 0;
   assign vras = 1;
   assign vcas = 1;
   assign vwe = 0;
   assign va[11:0] = 0;
   assign vd[15:0] = 0;
   assign vdqmh = 0;
   assign vdqml = 0;
   
   assign dvi_blue_p = 0;
   assign dvi_clock_p = 0;
   assign dvi_clock_n = 0;
   
dcm_clock dotclock_module (bclk, dotclock);
   

reg [15:0] registers [15:0];

wire [15:0] reg0 = registers[0];
wire [15:0] reg1 = registers[1];
wire [15:0] reg2 = registers[2];
wire [15:0] reg4 = registers[4];
wire [15:0] reg8 = registers[8];
   
   

reg [15:0]       trace_addr [15:0];
reg [15:0] 	 trace_data [15:0];
reg [8:0] 	 trace_count = 0;

wire [3:0] 	 trace_bit;
wire [3:0] 	 trace_get;
reg  [3:0] 	 trace_put = 0;
assign trace_get = trace_count[8:5];
assign trace_bit = trace_count[3:0];

reg [8:0] 	 trace_count_new;
reg [15:0] 	 trace_out_new;
reg 		 dvi_ddc_data_new;
reg 		 dvi_ddc_clock_new;
   
always @(trace_count or trace_put or trace_get
 or trace_addr[trace_count[8:5]] or trace_data[trace_count[8:5]] or trace_out_new[trace_count[3:0]])
  begin
     trace_count_new = trace_count;
     dvi_ddc_data_new = 0;
     dvi_ddc_clock_new = 0;

     case (trace_count[4])
       0: trace_out_new = trace_addr[trace_count[8:5]];
       1: trace_out_new = trace_data[trace_count[8:5]];
     endcase // case (trace_count[4])
     dvi_ddc_data_new = trace_out_new[trace_count[3:0]];
     if (trace_get != trace_put)
       begin
	  trace_count_new = trace_count + 1;
	  dvi_ddc_clock_new = 1;
       end
  end

always @(posedge bclk)
  begin
     trace_count <= trace_count_new;
     dvi_ddc_clock <= dvi_ddc_clock_new;
     dvi_ddc_data <= dvi_ddc_data_new;
  end

assign d_en = !cs4 && !rd;

reg been_there = 0;
   
reg [3:0] trace_put_new;

 always @(trace_put)
   begin
      trace_put_new = trace_put + 1;
   end

always @(posedge bclk)
  begin
     if (!cs4 && !wr0)
       begin
	  registers[a[3:0]] <= d;
	  trace_addr[trace_put] <=  { 4'b0, a[11:0] };
	  trace_data[trace_put] <= d;
	  trace_put <= trace_put_new;
       end
     if (!cs4 && !rd)
       begin
	  d_wr <= registers[a[3:0]];
	  if (!been_there)
	    begin
	       trace_addr[trace_put] <= { 4'b1111, a[11:0] };
	       trace_data[trace_put] <= registers[a[3:0]];
	       trace_put <= trace_put_new;
	       been_there = 1;
	    end
       end // if (!cs4 && !rd)
     else
       been_there = 0;
     
  end
   
   // These are the logic analyzer pins
   //
   // black	BLUE_N
   // brown	GREEN_N
   // red		RED_N
   // orange	RED_P
   // yellow	GREEN_P
   // green	DDC_CLOCK
   // blue		DDC_DATA
   // violet	HOTPLUG
   
   assign dvi_blue_n = bclk;
   assign dvi_green_n = dotclock;
   assign dvi_red_n = cs4;
   assign dvi_red_p = a[3];
   assign dvi_green_p = rd;
//   assign dvi_ddc_clock = wr0;
//   assign dvi_ddc_data = d[1];

endmodule // play1
