`timescale 1ns / 1ps

// clocked logic:
//    a <= 5;
//    b <= a;  b is the old value of a
//
// combinatorial logic:
//   a = 5;
//   b = a;   b is 5
//
// propagation delay:
//   assign #1 a = b;

module test1 ();

parameter tcyc = 10;
parameter tdly = 3;
  
reg  [7:0] state;
   

reg		bclk;
reg	[23:0]	a;
wire	[15:0]	d;
reg		cs3;
reg		cs4;
reg		rd;
reg		wr0;
reg		wr1;
wire		waitn;
reg		edack0;
wire		edreq0;

wire		vclk;
wire		vcke;
wire		vcs;
wire	[1:0]	vba;
wire		vras;
wire		vcas;
wire		vwe;
wire	[11:0]	va;
wire	[15:0]	vd;
wire		vdqmh;
wire		vdqml;

wire		dvi_red_n;
wire		dvi_red_p;
wire		dvi_green_n;
wire		dvi_green_p;
wire		dvi_blue_n;
wire		dvi_blue_p;
wire		dvi_clock_p;
wire		dvi_clock_n;
wire		dvi_ddc_clock;
wire		dvi_ddc_data;
wire		dvi_hotplug;

reg	[15:0]	d_val;
reg		d_en;
assign d = d_en ? d_val : 16'bz;
   

play1 dut(bclk,
	     a,
	     d,
	     cs3,
	     cs4,
	     rd,
	     wr0,
	     wr1,
	     waitn,
	     edack0,
	     edreq0,

	     vclk,
	     vcke,
	     vcs,
	     vba,
	     vras,
	     vcas,
	     vwe,
	     va,
	     vd,
	     vdqmh,
	     vdqml,

	     dvi_red_n,
	     dvi_red_p,
	     dvi_green_n,
	     dvi_green_p,
	     dvi_blue_n,
	     dvi_blue_p,
	     dvi_clock_p,
	     dvi_clock_n,
	     dvi_ddc_clock,
	     dvi_ddc_data,
	     dvi_hotplug,
	     );

initial
  begin
     while (1)
       begin
	  case ($time % 20)
	    0: bclk = 1;
	    10: bclk = 0;
	  endcase // case ($time % 20)
	  #1;
       end
  end

task automatic waitfor;
   input [7:0] mod;
   begin
      while (($time % 20) != mod)
	begin
	   #1;
	end
   end
endtask

task automatic bclk1;
   begin
      waitfor(3);
   end
endtask // bclk1

task automatic bclk0;
   begin
      waitfor(13);
   end
endtask // bclk1
   
task automatic mcu_tick;
   begin
      bclk0;
      bclk1;
   end
endtask // mcu_write


task ticks;
   begin
      bclk1;
      rd = 1;
      wr0 = 1;
      wr1 = 1;
      cs3 = 1;
      cs4 = 1;
      d_en = 0;

      state = 0;
      mcu_tick;
      mcu_tick;
   end
endtask // mcu_write

reg [7:0] cs = 0; // set according to address

reg [7:0] cson[7:0];

reg [7:0] csrwait[7:0];
reg [7:0] rdon[7:0];
reg [7:0] csroff[7:0];
   
reg [7:0] wdon[7:0];
reg [7:0] wron[7:0];
reg [7:0] cswwait[7:0];
reg [7:0] wdoff[7:0];
reg [7:0] cswoff[7:0];

initial
  begin
     cson[3] = 0;
     rdon[3] = 1;
     csrwait[3] = 5;
     csroff[3] = 1;
     wdon[3] = 0;
     wron[3] = 0;
     cswwait[3] = 0;
     wdoff[3] = 1;
     cswoff[3] = 1;
     
     cson[4] = 0;
     rdon[4] = 1;
     csrwait[4] = 5;
     csroff[4] = 1;
     wdon[4] = 0;
     wron[4] = 0;
     cswwait[4] = 0;
     wdoff[4] = 1;
     cswoff[4] = 1;

     cson[5] = 0;
     rdon[5] = 1;
     csrwait[5] = 2;
     csroff[5] = 1;
     wdon[5] = 0;
     wron[5] = 0;
     cswwait[5] = 5;
     wdoff[5] = 1;
     cswoff[5] = 1;
  end

task automatic twait;
   input [7:0] count;
   input [55:0]      name;
   begin
      while (count > 0)
	begin
	   count = count - 1;
	   mcu_tick;
	end
   end
endtask

reg mcu_cson_running = 0;
task mcu_cson;
   input [23:0] address;
   begin
      mcu_cson_running = 1;
      
      twait(cson[cs], "cson");
       case (address[23:20])
	 3: cs3 = 0;
	 4: cs4 = 0;
       endcase
      mcu_cson_running = 0;
   end
endtask

reg mcu_csrwait_running = 0;
task mcu_csrwait;
   input [23:0] address;
   begin
      mcu_csrwait_running = 1;
      twait(csrwait[cs], "csrwait");
      mcu_csrwait_running = 0;
   end
endtask

reg mcu_rdon_running = 0;
task mcu_rdon;
   input [23:0] address;
   begin
      mcu_rdon_running = 1;
      twait(rdon[cs], "rdon");
      rd = 0;
      mcu_rdon_running = 0;
   end
endtask
      
reg mcu_csroff_running = 0;
task mcu_csroff;
   input [23:0] address;
   begin
      mcu_csroff_running = 1;
      twait(csroff[cs], "csroff");
      cs3 = 1;
      cs4 = 1;
      mcu_csroff_running = 0;
   end
endtask
      
reg mcu_wdon_running = 0;
task mcu_wdon;
   input [23:0] address;
   begin
      mcu_wdon_running = 1;
      twait(wdon[cs], "wdon");
      d_en = 1;
      mcu_wdon_running = 0;
   end
endtask

reg mcu_wron_running = 0;
task mcu_wron;
   input [23:0] address;
   begin
      mcu_wron_running = 1;
      twait(wron[cs], "wron");
      wr0 = 0;
      wr1 = 0;
      mcu_wron_running = 0;
   end
endtask

reg mcu_cswwait_running = 0;
task mcu_cswwait;
   input [23:0] address;
   begin
      mcu_cswwait_running = 1;
      twait(cswwait[cs], "cswwait");
      mcu_cswwait_running = 0;
   end
endtask
      
reg mcu_wdoff_running = 0;
task mcu_wdoff;
   input [23:0] address;
   begin
      mcu_wdoff_running = 1;
      twait(wdoff[cs], "wdoff");
      d_en = 0;
      mcu_wdoff_running = 0;
   end
endtask
      
reg mcu_cswoff_running = 0;
task mcu_cswoff;
   input [23:0] address;
   begin
      mcu_cswoff_running = 1;
      twait(cswoff[cs], "cswoff");
      cs3 = 1;
      cs4 = 1;
      mcu_cswoff_running = 0;
   end
endtask

reg [7:0] mcu_state = 0;
   

task mcu_write;
   input [23:0] address;
   input [15:0] data;
   input [1:0] 	bhl;
   begin
      cs = address[23:20];

      bclk1;
      a = address;
      d_val = data;
      rd = 1;
      mcu_state = 1;
      fork
	 mcu_cson (address);
	 mcu_wron (address);
	 mcu_wdon (address);
	 mcu_cswwait (address);
      join
      mcu_state = 2;

      bclk0;
      bclk1;
      wr0 = 1;
      wr1 = 1;

      mcu_state = 3;
      fork
	 mcu_wdoff (address);
	 mcu_cswoff (address);
      join
      mcu_state = 4;
      address = 24'bz;
      data = 16'bz;
   end
endtask

reg [15:0] read_data;

task mcu_read;
   input [23:0] address;
   begin
      cs = address[23:20];

      bclk1;
      a = address;
      mcu_state = 5;
      fork
	 mcu_cson (address);
	 mcu_rdon (address);
	 mcu_csrwait (address);
      join
      mcu_state = 6;

      bclk0;
      bclk1;
      rd = 1;
      read_data = d;
      mcu_state = 7;
      fork
	 mcu_csroff (address);
      join
      mcu_state = 8;
      address = 24'bz;
   end
endtask

   integer i;
   
   
initial
  begin
     state = 0;
     bclk = 0;
     a = 0;
     cs3 = 1;
     cs4 = 1;
     rd = 1;
     wr0 = 1;
     wr1 = 1;
     edack0 = 0;

     d_en = 0;
     d_val = 0;
     
     $dumpvars;

     ticks();
     mcu_write (24'h500000, 16'h5555, 0);

     state = 2;
     mcu_write (24'h3000ff, 16'h00ff, 0);
     state = 3;
     mcu_write (24'h401234, 16'h55aa, 0);
     mcu_write (24'h401230, 16'h8765, 0);
     state = 4;
     mcu_write (24'h3000ff, 16'h00ff, 0);
     state = 5;

     mcu_read (24'h300000);
     state  = 10;
     mcu_read (24'h4000f4);
     mcu_read (24'h4000f0);
     state  = 11;
     mcu_read (24'h300000);

     mcu_write (24'h500000, 16'haaaa, 0);
     ticks();
     

     for (i=0; i<70; i=i+1)
       ticks();
     #10
     $finish;
     
  end

endmodule
